/* Java Modifiers */
/* See <http://java.sun.com/docs/books/jls/first_edition/html/19.doc.html#44488> */

%token IDENTIFIER
%token PUBLIC
%token PROTECTED
%token PRIVATE
%token STATIC
%token ABSTRACT
%token FINAL
%token NATIVE
%token SYNCHRONIZED
%token TRANSIENT
%token VOID
%token VOLATILE
%token Type VariableInitializer ThrowsOpt MethodBody ConstructorBody
%token FormalParameterListOpt

%start ClassBodyDeclaration

%%

SimpleTypeName:
  IDENTIFIER ;

/* 19.8.1 Productions from 8.1: Class Declaration */

ClassBodyDeclaration:
  ClassMemberDeclaration |
  ConstructorDeclaration ;

ClassMemberDeclaration:
  FieldDeclaration |
  MethodDeclaration ;

/* 19.8.2 Productions from 8.3: Field Declarations */

FieldDeclaration:
  FieldModifiersOpt Type VariableDeclarators ';' ;

VariableDeclarators:
  VariableDeclarator |
  VariableDeclarators ',' VariableDeclarator ;

VariableDeclarator:
  VariableDeclaratorId |
  VariableDeclaratorId '=' VariableInitializer ;

VariableDeclaratorId:
  IDENTIFIER ;

FieldModifiersOpt:
  /* empty */ |
  FieldModifiers ;

FieldModifiers:
  FieldModifier |
  FieldModifiers FieldModifier ;

FieldModifier:
  PUBLIC | PROTECTED | PRIVATE |
  FINAL | STATIC | TRANSIENT | VOLATILE ;

/* 19.8.3 Productions from 8.4: Method Declarations */

MethodDeclaration:
  MethodHeader MethodBody ;


MethodHeader:
  MethodModifiersOpt ResultType MethodDeclarator ThrowsOpt ;

ResultType:
  Type |
  VOID ;

MethodDeclarator:
  IDENTIFIER '(' FormalParameterListOpt ')' ;

MethodModifiersOpt:
  /* empty */ |
  MethodModifiers;

MethodModifiers:
  MethodModifier |
  MethodModifiers MethodModifier ;

MethodModifier:
  PUBLIC | PROTECTED | PRIVATE |
  ABSTRACT | STATIC | FINAL | SYNCHRONIZED | NATIVE ;

/* 19.8.5 Productions from 8.6: Constructor Declarations */

ConstructorDeclaration:
  ConstructorModifiersOpt ConstructorDeclarator ThrowsOpt ConstructorBody ;

ConstructorDeclarator:
  SimpleTypeName '(' FormalParameterListOpt ')' ;

ConstructorModifiersOpt:
  /* empty */ |
  ConstructorModifiers ;

ConstructorModifiers:
  ConstructorModifier |
  ConstructorModifiers ConstructorModifier ;

ConstructorModifier:
  PUBLIC | PROTECTED | PRIVATE ;
